@@ -60,9 +60,8 @@ namespace Tensile
             gfx906  = 906,
             gfx908  = 908,
             gfx90a  = 910,
-            gfx940  = 940,
-            gfx941  = 941,
             gfx942  = 942,
+            gfx950  = 950,
             gfx1010 = 1010,
             gfx1011 = 1011,
             gfx1012 = 1012,
@@ -73,7 +72,9 @@ namespace Tensile
             gfx1035 = 1035,
             gfx1100 = 1100,
             gfx1101 = 1101,
-            gfx1102 = 1102
+            gfx1102 = 1102,
+            gfx1150 = 1150,
+            gfx1151 = 1151
         };
 
         static std::string toString(Processor p)
@@ -90,12 +91,10 @@ namespace Tensile
                 return "gfx908";
             case AMDGPU::Processor::gfx90a:
                 return "gfx90a";
-            case AMDGPU::Processor::gfx940:
-                return "gfx940";
-            case AMDGPU::Processor::gfx941:
-                return "gfx941";
             case AMDGPU::Processor::gfx942:
                 return "gfx942";
+            case AMDGPU::Processor::gfx950:
+                return "gfx950";
             case AMDGPU::Processor::gfx1010:
                 return "gfx1010";
             case AMDGPU::Processor::gfx1011:
@@ -118,6 +117,10 @@ namespace Tensile
                 return "gfx1101";
             case AMDGPU::Processor::gfx1102:
                 return "gfx1102";
+            case AMDGPU::Processor::gfx1150:
+                return "gfx1150";
+            case AMDGPU::Processor::gfx1151:
+                return "gfx1151";
             }
             return "";
         }
@@ -144,18 +147,14 @@ namespace Tensile
             {
                 return AMDGPU::Processor::gfx90a;
             }
-            else if(deviceString.find("gfx940") != std::string::npos)
-            {
-                return AMDGPU::Processor::gfx940;
-            }
-            else if(deviceString.find("gfx941") != std::string::npos)
-            {
-                return AMDGPU::Processor::gfx941;
-            }
             else if(deviceString.find("gfx942") != std::string::npos)
             {
                 return AMDGPU::Processor::gfx942;
             }
+            else if(deviceString.find("gfx950") != std::string::npos)
+            {
+                return AMDGPU::Processor::gfx950;
+            }
             else if(deviceString.find("gfx1010") != std::string::npos)
             {
                 return AMDGPU::Processor::gfx1010;
@@ -184,6 +183,14 @@ namespace Tensile
             {
                 return AMDGPU::Processor::gfx1102;
             }
+            else if(deviceString.find("gfx1150") != std::string::npos)
+            {
+                return AMDGPU::Processor::gfx1150;
+            }
+            else if(deviceString.find("gfx1151") != std::string::npos)
+            {
+                return AMDGPU::Processor::gfx1151;
+            }
             else
             {
                 return static_cast<AMDGPU::Processor>(0);
@@ -207,6 +214,8 @@ namespace Tensile
         int         skDynamicGrid    = 0;
         int         skMaxCUs         = 0;
         int         skGridMultiplier = 1;
+        int         skFixedGrid      = 0;
+        int         skFullTiles      = 0;
         std::string deviceName;
 
         virtual bool   runsKernelTargeting(Processor p) const;
@@ -225,7 +234,7 @@ namespace Tensile
         const int getSKDynamicGrid() const
         {
             static const char* envStr = std::getenv("TENSILE_STREAMK_DYNAMIC_GRID");
-            static const int   value  = (envStr == NULL ? 0 : (std::atoi(envStr) == 0 ? 0 : 1));
+            static const int   value  = (envStr == NULL ? 0 : std::atoi(envStr));
             return value;
         }
 
@@ -243,6 +252,20 @@ namespace Tensile
             return value;
         }
 
+        const int getSKFixedGrid() const
+        {
+            static const char* envStr = std::getenv("TENSILE_STREAMK_FIXED_GRID");
+            static const int   value  = (envStr == NULL ? 0 : std::atoi(envStr));
+            return value;
+        }
+
+        const int getSKFullTiles() const
+        {
+            static const char* envStr = std::getenv("TENSILE_STREAMK_FULL_TILES");
+            static const int   value  = (envStr == NULL ? -1 : std::atoi(envStr));
+            return value;
+        }
+
         bool operator==(AMDGPU const& rhs) const
         {
             return processor == rhs.processor && computeUnitCount == rhs.computeUnitCount;
