@@ -135,7 +135,8 @@ namespace Tensile
         using HardwarePredicateSMT = SubclassMappingTraits<Predicates::Predicate<Hardware>, IO>;
 
         template <typename IO>
-        const typename HardwarePredicateSMT<IO>::SubclassMap HardwarePredicateSMT<IO>::subclasses
+        const typename HardwarePredicateSMT<IO>::SubclassMap
+            SubclassMappingTraits<Predicates::Predicate<Hardware>, IO>::subclasses
             = HardwarePredicateSMT<IO>::GetSubclasses();
 
         template <typename IO>
@@ -179,7 +180,8 @@ namespace Tensile
         using AMDGPUPredicateSMT = SubclassMappingTraits<Predicates::Predicate<AMDGPU>, IO>;
 
         template <typename IO>
-        const typename AMDGPUPredicateSMT<IO>::SubclassMap AMDGPUPredicateSMT<IO>::subclasses
+        const typename AMDGPUPredicateSMT<IO>::SubclassMap
+            SubclassMappingTraits<Predicates::Predicate<AMDGPU>, IO>::subclasses
             = AMDGPUPredicateSMT<IO>::GetSubclasses();
 
         template <typename IO>
@@ -218,9 +220,8 @@ namespace Tensile
                 iot::enumCase(io, value, "gfx906", AMDGPU::Processor::gfx906);
                 iot::enumCase(io, value, "gfx908", AMDGPU::Processor::gfx908);
                 iot::enumCase(io, value, "gfx90a", AMDGPU::Processor::gfx90a);
-                iot::enumCase(io, value, "gfx940", AMDGPU::Processor::gfx940);
-                iot::enumCase(io, value, "gfx941", AMDGPU::Processor::gfx941);
                 iot::enumCase(io, value, "gfx942", AMDGPU::Processor::gfx942);
+                iot::enumCase(io, value, "gfx950", AMDGPU::Processor::gfx950);
                 iot::enumCase(io, value, "gfx1010", AMDGPU::Processor::gfx1010);
                 iot::enumCase(io, value, "gfx1011", AMDGPU::Processor::gfx1011);
                 iot::enumCase(io, value, "gfx1012", AMDGPU::Processor::gfx1012);
@@ -232,6 +233,8 @@ namespace Tensile
                 iot::enumCase(io, value, "gfx1100", AMDGPU::Processor::gfx1100);
                 iot::enumCase(io, value, "gfx1101", AMDGPU::Processor::gfx1101);
                 iot::enumCase(io, value, "gfx1102", AMDGPU::Processor::gfx1102);
+                iot::enumCase(io, value, "gfx1150", AMDGPU::Processor::gfx1150);
+                iot::enumCase(io, value, "gfx1151", AMDGPU::Processor::gfx1151);
             }
         };
 
@@ -267,7 +270,8 @@ namespace Tensile
         using TaskPredicateSMT = SubclassMappingTraits<Predicates::Predicate<Task>, IO>;
 
         template <typename IO>
-        const typename TaskPredicateSMT<IO>::SubclassMap TaskPredicateSMT<IO>::subclasses
+        const typename TaskPredicateSMT<IO>::SubclassMap
+            SubclassMappingTraits<Predicates::Predicate<Task>, IO>::subclasses
             = TaskPredicateSMT<IO>::GetSubclasses();
 
         template <typename IO>
